/*********************************************************************
* TWI-Master
*
* Uwe Berger; 2008
*
**********************************************************************/
#ifndef F_CPU
	#define F_CPU 1000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"

#define LED_DDR  DDRB	
#define LED_PORT PORTB
#define LED_PIN  PB1

#define PRELOAD 	63000
#define ADDR_TINY 	0x20

typedef unsigned char uint8_t;

volatile uint8_t v1, v2, v3, v4;
volatile uint8_t hole_werte = 0;
	
//***********************************************************	
ISR(INT1_vect)
{
	// LED-PIN invertieren
	LED_PORT ^= (1 << LED_PIN); 		
	// Merker setzen
	hole_werte = 1;
}	
	
//***********************************************************	
void tiny_read(void)
{
	// Anstoss senden
	i2c_start(ADDR_TINY+I2C_WRITE);
	i2c_write(0);
	i2c_stop();
	// Werte lesen
	i2c_start(ADDR_TINY+I2C_READ);
	v1 = i2c_readAck();
	v2 = i2c_readAck();
	v3 = i2c_readAck();
	v4 = i2c_readNak();
	i2c_stop();
}

//***********************************************************	
//***********************************************************	
int main(void)
{
	char s[10];

	LED_DDR   = (1 << LED_PIN);			// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);		// LED ausschalten
	
	MCUCR |= (1<<ISC10) | (1<<ISC11);	// steigende Flanke (L->H) fuer INT1
	GICR  |= (1<<INT1);          		// externer Interrupt an INT1
	sei(); 								// Interrupts einschalten

	i2c_init();							// TWI initialisieren
	lcd_init();							// LCD initialisieren

	lcd_command(LCD_CLEAR);
	lcd_backlight(ON);
	lcd_printlc(1, 1, (unsigned char *)"-->BLIT-Board<--");
	lcd_printlc(2, 1, (unsigned char *)"Uwe Berger; 2008");
	lcd_wait_ms(1000);
	lcd_command(LCD_CLEAR);
	lcd_printlc(1, 1, (unsigned char *)"Tiny-Count.: ");

	while (true){
		// Interrupt wurde ausgeloest, also Werte via TWI holen/anzeigen
		if (hole_werte) {
			tiny_read();
			lcd_printlc(1, 14, (unsigned char *)"   ");
			itoa(v1, s, 10);
			lcd_printlr(1, 14, (unsigned char *)s);
			lcd_putcharlr(2, 1, v2 + '0');
			lcd_putcharlr(2, 5, v3 + '0');
			lcd_putcharlr(2, 10, v4 + '0');
			// Merker zuruecksetzen
			hole_werte = 0;
		}
	}					
}
