/*********************************************************************
* TWI-Slave (TWI via USI)
*
* Uwe Berger; 2008
*
**********************************************************************/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "USI_TWI_Slave.h"

// Sample TWI transmission commands
#define TWI_CMD_MASTER_WRITE 0x10
#define TWI_CMD_MASTER_READ  0x20
#define TWI_CMD_MASTER_WRITE_ALOT 0x30

// TWI-Slave-Adresse
#define TWI_SLAVE_ADDR	0x10				

// Preload-Wert des Timers (Blinker...)
#define PRELOAD			100

// Tiny-Board LED
#define LED_DDR  DDRB
#define LED_PORT PORTB
#define LED_PIN  PB3

// Tiny-Board INT-Ausgang
#define INT_DDR  DDRB
#define INT_PORT PORTB
#define INT_PIN  PB1

// Testwerte
#define VAL_MAX 4
volatile uint8_t val[VAL_MAX] = {0,2,3,4};
uint8_t val_index = 0;     
   

//*********************************************************************
ISR(TIM1_OVF_vect)
{
	// Preloadwert setzen
	TCNT1 = PRELOAD;					
	// LED-PIN invertieren
	LED_PORT ^= (1 << LED_PIN); 			
	// auf Interruptleitung Impuls erzeugen, also 2x invertieren
	INT_PORT ^= (1 << INT_PIN);             
	INT_PORT ^= (1 << INT_PIN);      
	// einen der Werte hochzaehlen
	val[0]++;       
}   
   
   
//*********************************************************************
//*********************************************************************
int main(void)
{
	uint8_t temp;

	DDRB  	 |= (1 << LED_PIN) | (1 << INT_PIN);	// LED- und INT-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);					// LED ausschalten
	INT_PORT &= ~(1 << INT_PIN);					// INT-Port auf LOW
	
	// Timer-Overflow-Interrupt setzen
	TCNT1   = PRELOAD;									// Preloadwert setzen
	TCCR1  |= (1 << CS13) | (1 << CS12) | (1 << CS11);	// Prescaler Timer1 auf 8192 stellen
	TIMSK  |= (1 << TOIE1);								// Timer 1 Overflow Interrupt enable

  
	// TWI initialisieren und Slave-Adresse setzen
	USI_TWI_Slave_Initialise( TWI_SLAVE_ADDR );

	sei();

	for(;;)	{

		// TWI-Verarbeitung
		if( USI_TWI_Data_In_Receive_Buffer() ) {
			// ein Byte vom Master empfangen 
			temp = USI_TWI_Receive_Byte();
			// und die Werte aus dem Testfeld senden
			USI_TWI_Transmit_Byte(val[val_index]);
			val_index++;
			USI_TWI_Transmit_Byte(val[val_index]);
			val_index++;
			USI_TWI_Transmit_Byte(val[val_index]);
			val_index++;
			USI_TWI_Transmit_Byte(val[val_index]);
			val_index = 0;
		}
	}
}

